import os
import matplotlib.pyplot as plt
from astropy.visualization import make_lupton_rgb
import astropy.units as u
from cutout.tools import objloc, getpanstarrsurl
from astropy.io import fits
from astropy.wcs import WCS
from unagi.task import hsc_cutout
from unagi import hsc



def decals(obj, wcsgrid = False, scalebar = False, labelimg = False, savepath = None, savefits = False):
	"""
	Return RGB DECaLS cutout from g, r, and z band imaging. If z-band is not available, will use g, r, and i
	instead (will print disclaimer).
	Pixel scale is 0.26"/pix.

	Parameters:
		obj (str): Name or coordinates for object of interest. If coordinates, should be in
			HH:MM:SS DD:MM:SS or degree formats. Names must be resolvable in SIMBAD.
		wcsgrid (bool): If True, show WCS grid on RGB image.
		scalebar (float): Length of scalebar in arcseconds. If specified, shown on image.
		labelimg (bool): If True, show obj string on image.
		savepath (str): Path specifying where to save image. If not specified, image is not saved.
		savefits (bool): If True, retain downloaded .fits file.

	Returns:
		If savepath, saves image to specified location. If savepath not specified, just displays image.
	"""

	coords = objloc(obj)
	fname = obj.replace(' ', '').replace(':','')+'.fits'
	lspath = '"https://www.legacysurvey.org/viewer/cutout.fits?ra='+str(coords.ra.deg)+'&dec='+str(coords.dec.deg)+'&layer=ls-dr9&pixscale=0.26&size=512"'
	os.system('curl -L '+lspath+' > "'+fname+'"')

	fig = plt.figure(figsize = (5, 5))

	img = fits.open(fname)

	g = img[0].data[0, :, :]
	r = img[0].data[1, :, :]
	z = img[0].data[2, :, :]

	if np.min(z) == np.max(z): # check for z-band data
		# redownload GRI data if z-band frame is empty
		lspath = lspath.replace('&size=512', '&size=512&bands=gri')
		os.system('curl -L '+lspath+' > "'+fname+'"')

		img = fits.open(fname)

		g = img[0].data[0, :, :]
		r = img[0].data[1, :, :]
		z = img[0].data[2, :, :]

		print('No z-band data available, output image will be gri instead.')

	rgb = make_lupton_rgb(0.75*z, 1.1*r, 1.75*g, stretch=0.1, Q=5)

	if wcsgrid:
		lswcs = WCS(img[0].header, img)
		plt.subplot(projection=lswcs.slice(view = [1]))
		plt.grid(color='gray', ls='dashed')
		plt.xlabel('RA')
		plt.ylabel('Dec')

	plt.imshow(rgb, origin = 'lower', interpolation = 'none')

	if not wcsgrid:
		plt.axis('off')

	if scalebar:
		x, y = g.shape
		npix = scalebar/0.26 #arcsec
		plt.plot([512/2 - npix/2, 512/2 + npix/2], [512/8]*2, color = 'white')
		plt.text(512/2 - 40, 512/8 + 5, s = '%i arcsec'%scalebar, color = 'white')

	if labelimg:
		plt.text(10, 480, s = obj, color = 'white')

	if savepath:
		fig.savefig(savepath, bbox_inches = 'tight', dpi = 200)

	plt.show()


	if not savefits:
		os.system('rm '+fname)



def hscssp(obj, wcsgrid = False, scalebar = False, labelimg = False, savepath = None, savefits = False):
	"""
	Return RGB HSC SSP cutout from g, r, and i band imaging.
	Pixel scale is 0.168"/pix.

	Parameters:
		obj (str): Name or coordinates for object of interest. If coordinates, should be in
			HH:MM:SS DD:MM:SS or degree formats. Names must be resolvable in SIMBAD.
		wcsgrid (bool): If True, show WCS grid on RGB image.
		scalebar (float): Length of scalebar in arcseconds. If specified, shown on image.
		labelimg (bool): If True, show obj string on image.
		savepath (str): Path specifying where to save image. If not specified, image is not saved.
		savefits (bool): If True, retain downloaded .fits file.

	Returns:
		If savepath, saves image to specified location. If savepath not specified, just displays image.
	"""

	coords = objloc(obj)
	fname = obj.replace(' ', '').replace(':', '')

	pdr = hsc.Hsc(dr='pdr2', rerun='any',config_file=None)

	g = hsc_cutout(coords, cutout_size=256*0.168*u.arcsec, filters='g', archive=pdr, save_output=savefits)
	r = hsc_cutout(coords, cutout_size=256*0.168*u.arcsec, filters='r', archive=pdr, save_output=savefits)
	i = hsc_cutout(coords, cutout_size=256*0.168*u.arcsec, filters='i', archive=pdr, save_output=savefits)
	

	fig = plt.figure(figsize = (5, 5))

	rgb = make_lupton_rgb(0.75*i[1].data, 1.*r[1].data, 1.5*g[1].data, stretch=0.5, Q=5)

	if wcsgrid:
		hscwcs = WCS(g[1].header)
		plt.subplot(projection=hscwcs)
		plt.grid(color='gray', ls='dashed')
		plt.xlabel('RA')
		plt.ylabel('Dec')

	plt.imshow(rgb, origin = 'lower', interpolation = 'none')

	if not wcsgrid:
		plt.axis('off')

	if scalebar:
		x, y = g[1].data.shape
		npix = scalebar/0.168 #arcsec
		plt.plot([512/2 - npix/2, 512/2 + npix/2], [512/8]*2, color = 'white')
		plt.text(512/2 - 40, 512/8 + 5, s = '%i arcsec'%scalebar, color = 'white')

	if labelimg:
		plt.text(10, 480, s = obj, color = 'white')

	if savepath:
		fig.savefig(savepath, bbox_inches = 'tight', dpi = 200)

	plt.show()

	if savefits:
		os.system('mv pdr2*_g.fits '+fname+'_g.fits')
		os.system('mv pdr2*_r.fits '+fname+'_r.fits')
		os.system('mv pdr2*_i.fits '+fname+'_i.fits')


def panstarrs(obj, wcsgrid = False, scalebar = False, labelimg = False, savepath = None, savefits = False):
	"""
	Return RGB Pan-STARRS1 cutout from g, r, and i band imaging.
	Pixel scale is 0.25"/pix.

	Parameters:
		obj (str): Name or coordinates for object of interest. If coordinates, should be in
			HH:MM:SS DD:MM:SS or degree formats. Names must be resolvable in SIMBAD.
		wcsgrid (bool): If True, show WCS grid on RGB image.
		scalebar (float): Length of scalebar in arcseconds. If specified, shown on image.
		labelimg (bool): If True, show obj string on image.
		savepath (str): Path specifying where to save image. If not specified, image is not saved.
		savefits (bool): If True, retain downloaded .fits file.

	Returns:
		If savepath, saves image to specified location. If savepath not specified, just displays image.
	"""

	coords = objloc(obj)
	fname = obj.replace(' ', '').replace(':','')
	psurl = getpanstarrsurl(coords.ra.deg, coords.dec.deg)

	if savefits:
		for i, f in enumerate(['i', 'r', 'g']):
			os.system('wget -O '+fname+'_%s.fits'%f + ' "'+psurl[i]+'"')

	fig = plt.figure(figsize = (5, 5))

	g_ = fits.open(psurl[2])
	r_ = fits.open(psurl[1])
	i_ = fits.open(psurl[0])

	rgb = make_lupton_rgb(0.5*i_[0].data, 0.65*r_[0].data, 1.*g_[0].data, stretch=500, Q=8)

	if wcsgrid:
		pswcs = WCS(g_[0].header, g_)
		plt.subplot(projection=pswcs)
		plt.grid(color='gray', ls='dashed')
		plt.xlabel('RA')
		plt.ylabel('Dec')

	plt.imshow(rgb, origin = 'lower', interpolation = 'none')

	if not wcsgrid:
		plt.axis('off')

	if scalebar:
		x, y = g_[0].data.shape
		npix = scalebar/0.25 #arcsec
		plt.plot([512/2 - npix/2, 512/2 + npix/2], [512/8]*2, color = 'white')
		plt.text(512/2 - 40, 512/8 + 5, s = '%i arcsec'%scalebar, color = 'white')

	if labelimg:
		plt.text(10, 480, s = obj, color = 'white')

	if savepath:
		fig.savefig(savepath, bbox_inches = 'tight', dpi = 200)

	plt.show()





# add https://skymapper.anu.edu.au/how-to-access/?

# add DELVE (and maybe other things from NOIRLab?): 
# https://github.com/astro-datalab/notebooks-latest/blob/master/04_HowTos/SiaService/How_to_use_the_Simple_Image_Access_service.ipynb








import setuptools
from pathlib import Path
cutout_home = Path(__file__).parent
pypi_descrip = (cutout_home / "README.md").read_text()

setuptools.setup(
	name = "cutout",
	version = "0.3.1",
	author = "Ava Polzin",
	author_email = "apolzin@uchicago.edu",
	description = "Survey cutouts plotted directly; no user choices, just plotting!",
	packages = ["cutout", "cutout/survey", "cutout/tools"],
	url = "https://github.com/avapolzin/cutout",
	license = 'MIT',
	classifiers = [
		"Development Status :: 4 - Beta",
		"Intended Audience :: Science/Research",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
		"Programming Language :: Python",
		"Topic :: Scientific/Engineering :: Astronomy",
		"Topic :: Scientific/Engineering :: Physics"],
	python_requires = ">=3",
	install_requires = ["astropy", "matplotlib", "numpy", "unagi"],
	long_description=pypi_descrip,
    long_description_content_type='text/markdown'
)

#more updated (development version of) unagi can be installed by replacing
# "unagi" with "unagi @ git+https://github.com/dr-guangtou/unagi"


from astropy.coordinates import SkyCoord, name_resolve
from astropy.table import Table
import astropy.units as u
import numpy as np

def objloc(obj):
	"""
	Get object location.

	Parameters:
		obj (str): Name or coordinates for object of interest. If coordinates, should be in
			HH:MM:SS DD:MM:SS or degree formats.
	Outputs:
		coords (astropy coordinates object)
	"""
	isname = False #check if obj is name or coordinates
	for s in obj:
		if s.isalpha():
			isname = True
			break

	if isname:
		coords = name_resolve.get_icrs_coordinates(obj)

	if not isname:
		if ':' in obj:
			coords = SkyCoord(obj, unit = [u.hour, u.deg])
		if not ':' in obj:
			coords = SkyCoord(obj, unit = u.deg)

	return coords


def getpanstarrsimages(ra,dec,filters="gri"):
	### COPIED FROM https://ps1images.stsci.edu/ps1image.html

	"""Query ps1filenames.py service to get a list of images

	ra, dec = position in degrees
	size = image size in pixels (0.25 arcsec/pixel)
	filters = string with filters to include
	Returns a table with the results
	"""

	service = "https://ps1images.stsci.edu/cgi-bin/ps1filenames.py"
	url = f"{service}?ra={ra}&dec={dec}&filters={filters}"
	table = Table.read(url, format='ascii')
	return table


def getpanstarrsurl(ra, dec, size=512, output_size=None, filters="gri", format="fits", color=False):
	### COPIED FROM https://ps1images.stsci.edu/ps1image.html
	
	"""Get URL for images in the table
	
	ra, dec = position in degrees
	size = extracted image size in pixels (0.25 arcsec/pixel)
	output_size = output (display) image size in pixels (default = size).
					output_size has no effect for fits format images.
	filters = string with filters to include
	format = data format (options are "jpg", "png" or "fits")
	color = if True, creates a color image (only for jpg or png format).
			Default is return a list of URLs for single-filter grayscale images.
	Returns a string with the URL
	"""
	
	if color and format == "fits":
		raise ValueError("color images are available only for jpg or png formats")
	if format not in ("jpg","png","fits"):
		raise ValueError("format must be one of jpg, png, fits")
	table = getpanstarrsimages(ra,dec,filters=filters)
	url = (f"https://ps1images.stsci.edu/cgi-bin/fitscut.cgi?"
		   f"ra={ra}&dec={dec}&size={size}&format={format}")
	if output_size:
		url = url + "&output_size={}".format(output_size)
	# sort filters from red to blue
	flist = ["yzirg".find(x) for x in table['filter']]
	table = table[np.argsort(flist)]
	if color:
		if len(table) > 3:
			# pick 3 filters
			table = table[[0,len(table)//2,len(table)-1]]
		for i, param in enumerate(["red","green","blue"]):
			url = url + "&{}={}".format(param,table['filename'][i])
	else:
		urlbase = url + "&red="
		url = []
		for filename in table['filename']:
			url.append(urlbase+filename)
	return url
